@@ -100,6 +100,7 @@ sub _create {
 				      _dcc_read
 				      _dcc_timeout
 				      _dcc_up
+				      _delayed_cmd
 				      _parseline
 				      __send_event
 				      _sock_down
@@ -221,6 +222,7 @@ sub qnet_bot_commands {
   }
   $command = join(' ',$command,$message) if ( defined ( $message ) );
   $kernel->yield( 'sl_prioritized', $pri, $command );
+  undef;
 }
 
 sub service_bots {
